#include "include/fs.h"
#include "include/type.h"
#include "include/string.h"
#include "include/disk.h"
#include "include/buf.h"
#include "include/panic.h"
#include "include/assert.h"

extern void virtio_disk_rw(struct buf *b, int write);
static struct superblock sb;
static struct inode inodes[MAX_FILES];

void fs_init(void) {
    memset(&sb, 0, sizeof(sb));
    memset(inodes, 0, sizeof(inodes));
}

int fs_open(const char *path, int mode) {
    for (int i = 0; i < MAX_FILES; i++) {
        if (inodes[i].used && strcmp(inodes[i].name, path) == 0) {
            return i;
        }
    }
    return -1;
}

int fs_close(int fd) {
    for (int i = 0; i < MAX_BLOCKS; i++) {
        if (inodes[fd].blocks[i] != 0) {
            struct buf b;
            b.blockno = inodes[fd].blocks[i];
            virtio_disk_rw(&b, 1);
        }
    }
    return 0;
}

int fs_read(int fd, void *buf, uint64_t len) {
    int n = 0;
    for (int i = 0; i < MAX_BLOCKS && n < len; i++) {
        if (inodes[fd].blocks[i] == 0) {
            break;
        }
        struct buf b;
        b.blockno = inodes[fd].blocks[i];
        virtio_disk_rw(&b, 0);
        memmove(buf + n, b.data, BSIZE);
        n += BSIZE;
    }
    return n;
}

int fs_write(int fd, const void *buf, uint64_t len) {
    TODO();
    return 0;
}

int fs_mount(const char* device) {
    struct buf b;
    b.blockno = 1;
    virtio_disk_rw(&b, 0);
    memmove(&sb, b.data, sizeof(sb));
    b.blockno = 2;
    virtio_disk_rw(&b, 0);
    memmove(inodes, b.data, sizeof(inodes));
    Assert(sb.magic == 0x11111111, "Invalid magic number");

    b.blockno = 4;
    virtio_disk_rw(&b, 0);
    printk("%s\n", b.data);
    return 0;
}

void fs_sync(void) {
    TODO();
}

int fs_mkdir(const char *path) {

    TODO();
    return 0;
}

int fs_chdir(const char *path) {
    TODO();
    return 0;
}

int fs_readdir(int fd, struct dirent *dir) {
    TODO();
    return 0;
}

char* fs_getcwd(char *buf, uint64_t size) {
    TODO();
    return NULL;
}

