#ifndef FS_H
#define FS_H

#include "type.h"

#define O_RDONLY  0x000
#define O_WRONLY  0x001
#define O_RDWR    0x002
#define O_CREATE  0x200

#define T_FILE 0
#define T_DIR  1


#define MAX_FILES 10
#define MAX_FILE_NAME 16
#define MAX_FILE_SIZE 4096

#define BLOCK_SIZE 1024
#define MAX_BLOCKS 8

struct superblock {
    uint32_t magic;
    uint32_t block_num;
    uint32_t inode_num;
    uint32_t size;
};

struct inode {
    char name[MAX_FILE_NAME];
    uint8_t used;
    uint8_t type;
    uint32_t size;
    uint32_t blocks[MAX_BLOCKS];
};

struct dirent {
    char name[MAX_FILE_NAME];
    uint8_t inode_num;
};

void fs_init(void);
int fs_open(const char *path, int mode);
int fs_close(int fd);
int fs_read(int fd, void *buf, uint64_t len);
int fs_write(int fd, const void *buf, uint64_t len);
int fs_mount(const char* device);
void fs_sync(void);
int fs_mkdir(const char *path);
int fs_chdir(const char *path);
int fs_readdir(int fd, struct dirent *dir);
char* fs_getcwd(char *buf, uint64_t size);

#endif
