#ifndef PROC_H
#define PROC_H
#include "type.h"
#include "riscv.h"
#include "vm.h"

#define MAX_PROC_NUM 16

#define DEFAULT_PRIORITY 1

#define USER_STACK_TOP    0x7FFFFFFFF000UL
#define USER_STACK_SIZE   PAGE_SIZE
#define USER_STACK_BASE   (USER_STACK_TOP - USER_STACK_SIZE)

// jsut for test, may be changed
#define USER_CODE_BASE    0x1000000UL
#define USER_HEAP_BASE    0x2000000UL

#define PROC_KERNEL_STACK_BASE    0x90000000UL

#define KERNEL_STACK_SIZE     PAGE_SIZE
#define KERNEL_STACK_PAGES    4
#define KERNEL_STACK_SHIFT    14

typedef struct Context {
    uint64_t gpr[31];
    uint64_t sepc;
    uint64_t sstatus;
    uint64_t satp;
    uint64_t scause;
}Context;

typedef enum proc_state {
    PROC_UNUSED,
    PROC_EMBRYO,
    PROC_SLEEPING,
    PROC_RUNNABLE,
    PROC_RUNNING,
    PROC_ZOMBIE,
}State;

typedef struct ProcessControlBlock {
    uint64_t *stack;
    uint64_t *kstack;
    uint64_t pid;
    State state;
    Context *context;
    struct ProcessControlBlock *parent;
    uint8_t priority;
    pagetable_t *pagetable;
    void* chan;      // channel for sleep
    uint64_t kstack_va;
}PCB;

void init_proc();
PCB* alloc_pcb();
void yield();
void fork();
void exit();
void wait();
void sleep(void* chan);
void wakeup(void* chan);
void schedule();
int alloc_pid();
void switch_context(Context** old_ctx, Context* new_ctx);

#endif
