#ifndef VM_H
#define VM_H

#include "type.h"
#include "riscv.h"

#define PTE_V 0x001 // valid
#define PTE_R 0x002 // read
#define PTE_W 0x004 // write
#define PTE_X 0x008 // execute
#define PTE_U 0x010 // user

#define SATP_MODE_SV39 8UL
#define VA_BITS 39
#define PTE_PPN_SHIFT 10
#define PTE_SIZE 8

#define VPN_2(va) (((uint64_t)(va) >> 30) & 0x1FF)
#define VPN_1(va) (((uint64_t)(va) >> 21) & 0x1FF)
#define VPN_0(va) (((uint64_t)(va) >> 12) & 0x1FF)

typedef uint64_t pte_t;
#define PTE_NUM 512
#define PTE_FLAGS_MASK 0x3FF
#define PA_TO_PTE(pa) (((uint64_t)(pa) >> 12) << 10)
#define PTE_TO_PA(pte) (((uint64_t)(pte) >> 10) << 12)

typedef struct {
    pte_t entries[512];
} pagetable_t;

void init_pagetable(pagetable_t* pt);
pagetable_t* create_pagetable();
int free_pagetable(pagetable_t* pt, int level);
int map_page(pagetable_t* root, uint64_t va, uint64_t pa, uint32_t perm);
int unmap_page(pagetable_t* root, uint64_t va);
uint64_t walk(pagetable_t* root, uint64_t va);
pte_t* walk_pte(pagetable_t* root, uint64_t va, int alloc);
pagetable_t* copy_pagetable(pagetable_t* src);
int map_range(pagetable_t* pt, uint64_t va, uint64_t pa, uint64_t size, uint32_t perm);
int unmap_range(pagetable_t* pt, uint64_t va, uint64_t size);
int check_permission(pagetable_t* pt, uint64_t va, uint32_t perm);

inline uint64_t get_pa(pte_t pte) {
    return PTE_TO_PA(pte);
}
inline int get_perm(pte_t pte) {
    return pte & PTE_FLAGS_MASK;
}

#endif