#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <stdint.h>
#include "../kernel/include/fs.h"

struct superblock sb;
struct inode inodes[MAX_FILES];
static uint64_t num = 3;
#define inode_size  sizeof(struct inode)

void write_block(int fd, const char *buf) {
    if (write(fd, buf, BLOCK_SIZE) != BLOCK_SIZE) {
        perror("write failed");
        exit(1);
    }
}
void add_file(const char *filename, int img_fd) {
    FILE *fp = fopen(filename, "rb");
    if (!fp) {
        perror("fopen");
    }

    fseek(fp, 0, SEEK_END);
    long file_size = ftell(fp);
    fseek(fp, 0, SEEK_SET);

    if (file_size > MAX_BLOCKS * BLOCK_SIZE) {
        fprintf(stderr, "File too large: %s (max: %d bytes)\n", 
                filename, MAX_BLOCKS * BLOCK_SIZE);
        fclose(fp);
    }
    int num_blocks = (file_size) / BLOCK_SIZE + ((file_size) % BLOCK_SIZE == 0 ? 0 : 1);

    char buf[BLOCK_SIZE];
    
    for (int i = 0; i < num_blocks; i++) {
        memset(buf, 0, BLOCK_SIZE);
        int n = fread(buf, 1, BLOCK_SIZE, fp);
        if (n < 0) {
            perror("fread");
            fclose(fp);
        }
        write_block(img_fd, buf);
    }

    fclose(fp);
}
void pre_add_file(const char *filename) {
    FILE *fp = fopen(filename, "rb");
    if (!fp) {
        perror("fopen");
        return;
    }

    fseek(fp, 0, SEEK_END);
    long file_size = ftell(fp);
    fseek(fp, 0, SEEK_SET);

    if (file_size > MAX_BLOCKS * BLOCK_SIZE) {
        fprintf(stderr, "File too large: %s (max: %d bytes)\n", 
                filename, MAX_BLOCKS * BLOCK_SIZE);
        fclose(fp);
        return;
    }

    int inum;
    for (inum = 0; inum < MAX_FILES; inum++) {
        if (!inodes[inum].used) {
            sb.inode_num++;
            break;
        }
    }
    inodes[inum].used = 1;
    inodes[inum].type = T_FILE;
    inodes[inum].size = file_size;
    strcpy(inodes[inum].name, filename);
    int block_num = (file_size) / BLOCK_SIZE + ((file_size) % BLOCK_SIZE == 0 ? 0 : 1);
    for (int i = 0; i < block_num; i++) {
        inodes[inum].blocks[i] = ++num;
    }
    fclose(fp);
}

void write_inode(int fd, struct inode inode) {
    char buf[inode_size];
    memset(buf, 0, inode_size);
    memcpy(buf, &inode, inode_size);
    write_block(fd, buf);
}

int main(int argc, char *argv[]) {
    if (argc < 2) {
        fprintf(stderr, "Usage: %s fs.img [files...]\n", argv[0]);
        exit(1);
    }

    int fd = open(argv[1], O_CREAT | O_RDWR | O_TRUNC, 0666);
    if (fd < 0) {
        perror("open failed");
        exit(1);
    }

    char *buf = (char *)calloc(1, BLOCK_SIZE);
    memset(&sb, 0, sizeof(sb));
    sb.magic = 0x11111111;
    sb.block_num = 2048;
    memset(inodes, 0, sizeof(inodes));
    for (int i = 2; i < argc; i++) {
        pre_add_file(argv[i]);
    }
    memset(buf, 0, BLOCK_SIZE); 
    write_block(fd, buf);
    memset(buf, 0, BLOCK_SIZE);
    memcpy(buf, &sb, sizeof(sb));
    write_block(fd, buf);
    memset(buf, 0, BLOCK_SIZE);
    memcpy(buf, inodes, sizeof(inodes));
    write_block(fd, buf);
    
    for (int i = 2; i < argc; i++) {
        memset(buf, 0, BLOCK_SIZE);
        add_file(argv[i], fd);
    }


    memset(buf, 0, BLOCK_SIZE);

    for (int i = 2048 - num; i > 0; i--) {
        write_block(fd, buf);
    }

    free(buf);
    close(fd);

    return 0;
}


